@@ -2,11 +2,12 @@
 import argparse
 import asyncio
 import logging
+import math
 import time
 
 import aiohttp
+import arrow
 from outdated import check_outdated
-from rich import print as rprint
 from tqdm import tqdm
 
 from . import __display_name__
@@ -17,6 +18,7 @@ from .collect import get_secrets
 from .collect import LinkedMediaHelper
 from .collect import RedditHelper
 from .control import Configuration
+from .control import PostRecorder
 from .monitoring import HealthChecks
 from .publish import MastodonPublisher
 
@@ -31,6 +33,7 @@ async def main() -> None:
     :return: None
     """
     # pylint: disable-msg=too-many-locals
+    # pylint: disable-msg=too-many-statements
     parser = argparse.ArgumentParser(description="Delete old toots.")
     parser.add_argument(
         "-c",
@@ -55,7 +58,7 @@ async def main() -> None:
         debug_log=args.debug_log_file,
     )
 
-    rprint(f"Welcome to {__display_name__} ({__version__})")
+    print(f"Welcome to {__display_name__} ({__version__})")
     check_updates()
 
     secrets = await get_secrets(config_dir=config_dir)
@@ -89,11 +92,18 @@ async def main() -> None:
 
         media_helper = LinkedMediaHelper(
             config=config,
-            gfycat_secrets=secrets["gfycat"],
             imgur_secrets=secrets["imgur"],
         )
         progress_bar.update(1 - 0.4130)
 
+    now_timestamp = arrow.now().timestamp()
+    last_post_ts = await config.bot.post_recorder.get_setting(PostRecorder.LAST_POST_TS)
+
+    if now_timestamp - last_post_ts < config.bot.delay_between_posts:
+        sleep_time = last_post_ts + config.bot.delay_between_posts - now_timestamp
+        bar_title = "Sleeping until next toot"
+        sleep_before_next_toot(bar_title, math.ceil(sleep_time))
+
     # Run the main script
     while True:
         if config.health.enabled:
@@ -106,6 +116,11 @@ async def main() -> None:
         if config.health.enabled:
             await healthcheck.check_ok()
 
+        await config.bot.post_recorder.save_setting(
+            PostRecorder.LAST_POST_TS,
+            arrow.now().timestamp(),
+        )
+
         if config.bot.run_once_only:
             logger.debug(
                 "Exiting because RunOnceOnly is set to %s", config.bot.run_once_only
@@ -127,27 +142,36 @@ async def main() -> None:
                 sleep_time = config.mastodon_config.throttling_max_delay
 
         logger.debug("Sleeping for %s seconds", sleep_time)
-
-        rprint(" ")
+        print(" ")
         bar_title = "Sleeping before next toot"
-        with tqdm(
-            desc=f"{bar_title:.<60}",
-            total=sleep_time,
-            unit="s",
-            ncols=120,
-            bar_format=PROGRESS_BAR_FORMAT,
-        ) as progress_bar:
-            for _i in range(sleep_time):
-                time.sleep(1)
-                progress_bar.update()
-                # progress_bar()  # pylint: disable=not-callable
+        sleep_before_next_toot(bar_title, sleep_time)
 
-        rprint(" ")
+        print(" ")
         logger.debug("Restarting main process...")
 
     await session.close()
 
 
+def sleep_before_next_toot(bar_title: str, sleep_time: int) -> None:
+    """Sleeps with progress bar for "sleep_time" seconds.
+
+    :param bar_title: Message to show in progress bar.
+    :type bar_title: str
+    :param sleep_time: Number of seconds to sleep
+    :type sleep_time: int
+    """
+    with tqdm(
+        desc=f"{bar_title:.<60}",
+        total=sleep_time,
+        unit="s",
+        ncols=120,
+        bar_format=PROGRESS_BAR_FORMAT,
+    ) as progress_bar:
+        for _i in range(sleep_time):
+            time.sleep(1)
+            progress_bar.update()
+
+
 def check_updates() -> None:
     """Check if there is a newer version of MastodonAmnesia available on
     PyPI."""
@@ -158,14 +182,12 @@ def check_updates() -> None:
             version=__version__,
         )
         if is_outdated:
-            rprint(
-                f"[bold][red]!!! New version of Tootbot ({pypi_version}) "
+            print(
+                f"!!! New version of Tootbot ({pypi_version}) "
                 f"is available on PyPI.org !!!\n"
             )
     except ValueError:
-        rprint(
-            "[yellow]Notice - Your version is higher than last published version on PyPI"
-        )
+        print("Notice - Your version is higher than last published version on PyPI")
 
 
 def start_main() -> None:
