@@ -22,15 +22,11 @@ import aiofiles
 import aiohttp
 import asyncpraw.exceptions
 import asyncprawcore
+import magic
 from asyncpraw.models import Submission
 from bs4 import BeautifulSoup
-from gfycat.client import GfycatClient
-from gfycat.error import GfycatClientError
 from imgurpython import ImgurClient
 from imgurpython.helpers.error import ImgurClientError
-from PIL import Image as PILImage
-from PIL import UnidentifiedImageError
-from rich import print as rprint
 from tqdm import tqdm
 from tqdm.asyncio import tqdm as aiotqdm
 
@@ -61,7 +57,12 @@ async def get_file(img_url: str, file_path: str, progress_label: str) -> Optiona
     Returns:
         file_path (string): path to downloaded image or None if no image was downloaded
     """
-    logger.debug("!!! Async Get File %s from %s", file_path, img_url)
+    logger.debug(
+        "collect.py - get_file(img_url=%s, file_path=%s, progress_label=%s)",
+        img_url,
+        file_path,
+        progress_label,
+    )
     chunk_size = 64 * 1024
     try:
         client = aiohttp.ClientSession(raise_for_status=True)
@@ -70,13 +71,13 @@ async def get_file(img_url: str, file_path: str, progress_label: str) -> Optiona
 
         file_out = await aiofiles.open(file=file_path, mode="wb")
         progress_bar = tqdm(
-            unit="iB",
+            unit="B",
             unit_scale=True,
             unit_divisor=1024,
             desc=f"{progress_label:.<60}",
             ncols=120,
-            bar_format=PROGRESS_BAR_FORMAT,
             total=download_size,
+            leave=True,
         )
         response = await client.get(url=img_url)
         async for data_chunk in response.content.iter_chunked(chunk_size):
@@ -86,10 +87,14 @@ async def get_file(img_url: str, file_path: str, progress_label: str) -> Optiona
         await file_out.close()
         await client.close()
         await asyncio.sleep(0)  # allow client session to close before continuing
+        logger.debug("collect.py - get_file(...) -> file_path=%s", file_path)
         return file_path
 
     except aiohttp.ClientError as save_image_error:
-        logger.error("Failed to download (%s): %s", img_url, save_image_error)
+        logger.error(
+            "collect.py - get_file(...) -> None - download failed with: %s",
+            save_image_error,
+        )
         return None
 
 
@@ -99,7 +104,6 @@ async def get_secrets(config_dir: str) -> Dict[str, Secret]:
 
     secrets = {}
     secrets["reddit"] = await _get_reddit_secret(config_dir=config_dir)
-    secrets["gfycat"] = _get_gfycat_secret(config_dir=config_dir)
     secrets["imgur"] = _get_imgur_secret(config_dir=config_dir)
 
     return secrets
@@ -111,8 +115,8 @@ async def _get_reddit_secret(config_dir: str) -> Secret:
     reddit_secrets_file = f"{config_dir}/reddit.secret"  # pragma: allowlist secret
     reddit_config = configparser.ConfigParser()
     if not os.path.exists(reddit_secrets_file):
-        rprint("Reddit API keys not found. Please provide Reddit API values.")
-        rprint("(See wiki if you need help).")
+        print("Reddit API keys not found. Please provide Reddit API values.")
+        print("(See wiki if you need help).")
         # Whitespaces are stripped from input: https://stackoverflow.com/a/3739939
         reddit_agent = "".join(input("[ .. ] Enter Reddit agent: ").split())
         reddit_client_secret = "".join(
@@ -152,57 +156,6 @@ async def _get_reddit_secret(config_dir: str) -> Secret:
     )
 
 
-def _get_gfycat_secret(config_dir: str) -> Secret:
-    """_get_gfycat_secrets checks if the Gfycat api secrets file exists.
-
-    - If the file exists, this method reads the files and returns the secrets
-      in a Secret dataclass.
-    - If the file doesn't exist it asks the user over stdin to supply these values
-      and then saves them into the gfycat_secrets file
-
-    Returns:
-        api_secrets (Secret): instance of Secret class containing the api secrets
-        to work with gyfycat
-    """
-
-    secrets_file = f"{config_dir}/gfycat.secret"  # pragma: allowlist secret
-    if not os.path.exists(secrets_file):
-        rprint("Gfycat API keys not found. (See wiki if you need help).")
-
-        # Whitespaces are stripped from input: https://stackoverflow.com/a/3739939
-        gfycat_client_id = "".join(input("[ .. ] Enter Gfycat client ID: ").split())
-        gfycat_client_secret = "".join(
-            input("[ .. ] Enter Gfycat client secret: ").split()
-        )
-        # Make sure authentication is working
-        try:
-            gfycat_client = GfycatClient(gfycat_client_id, gfycat_client_secret)
-
-            # If this call doesn't work, it'll throw an ImgurClientError
-            gfycat_client.query_gfy("oddyearlyhorsefly")
-            # It worked, so save the keys to a file
-            gfycat_config = configparser.ConfigParser()
-            gfycat_config["Gfycat"] = {
-                "ClientID": gfycat_client_id,
-                "ClientSecret": gfycat_client_secret,
-            }
-            with open(secrets_file, "w", encoding="utf8") as file:
-                gfycat_config.write(file)
-        except GfycatClientError as gfycat_error:
-            logger.error("Error while logging into Gfycat: %s", gfycat_error)
-            logger.error(FATAL_TOOTBOT_ERROR)
-            sys.exit(1)
-    else:
-        # Read API keys from secret file
-        gfycat_config = configparser.ConfigParser()
-        gfycat_config.read(secrets_file)
-
-    return Secret(
-        client_id=gfycat_config["Gfycat"]["ClientID"],
-        client_secret=gfycat_config["Gfycat"]["ClientSecret"],
-    )
-
-
 def _get_imgur_secret(config_dir: str) -> Secret:
     """_get_imgur_secrets checks if the Imgur api secrets file exists.
 
@@ -217,7 +170,7 @@ def _get_imgur_secret(config_dir: str) -> Secret:
     """
     secrets_file = f"{config_dir}/imgur.secret"  # pragma: allowlist secret
     if not os.path.exists(secrets_file):
-        rprint("Imgur API keys not found. (See wiki if you need help).")
+        print("Imgur API keys not found. (See wiki if you need help).")
 
         # Whitespaces are stripped from input: https://stackoverflow.com/a/3739939
         imgur_client_id = "".join(input("[ .. ] Enter Imgur client ID: ").split())
@@ -426,7 +379,6 @@ class LinkedMediaHelper:
     def __init__(
         self: LMH,
         config: Configuration,
-        gfycat_secrets: Secret,
         imgur_secrets: Secret,
     ) -> None:
         self.save_dir = config.media.folder
@@ -436,18 +388,10 @@ class LinkedMediaHelper:
                 client_id=imgur_secrets.client_id,
                 client_secret=imgur_secrets.client_secret,
             )
-            self.gfycat_client = GfycatClient(
-                client_id=gfycat_secrets.client_id,
-                client_secret=gfycat_secrets.client_secret,
-            )
         except ImgurClientError as imgur_error:
             logger.error("Error on creating ImgurClient: %s", imgur_error)
             logger.error(FATAL_TOOTBOT_ERROR)
             sys.exit(1)
-        except GfycatClientError as gfycat_error:
-            logger.error("Error on creating GfycatClient: %s", gfycat_error)
-            logger.error(FATAL_TOOTBOT_ERROR)
-            sys.exit(1)
 
     async def get_imgur_image(
         self: LMH, img_url: str, max_images: int = 4
@@ -506,15 +450,12 @@ class LinkedMediaHelper:
                 progress_label="Downloading Imgur image",
             )
 
-            try:
-                # Imgur will sometimes return a single-frame thumbnail
-                # instead of a GIF, so we need to check for this
-                if saved_paths and (
-                    file_extension != ".gif" or self._check_imgur_gif(saved_paths)
-                ):
-                    imgur_paths.append(saved_paths)
-            except UnidentifiedImageError:
-                continue
+            # Imgur will sometimes return a single-frame thumbnail
+            # instead of a GIF, so we need to check for this
+            if saved_paths and (
+                file_extension != ".gif" or self._check_imgur_gif(saved_paths)
+            ):
+                imgur_paths.append(saved_paths)
 
             if len(imgur_paths) == max_images:
                 break
@@ -557,10 +498,7 @@ class LinkedMediaHelper:
         Returns:
              True if downloaded image is indeed a GIF, otherwise returns False
         """
-        img = PILImage.open(file_path)
-        mime = PILImage.MIME[img.format]
-        img.close()
-
+        mime = magic.from_file(filename=file_path, mime=True)
         if mime != "image/gif":
             logger.warning("Imgur: not a GIF, not posting")
             try:
@@ -581,22 +519,45 @@ class LinkedMediaHelper:
             file_path (string): path to downloaded image or None if no image was
             downloaded
         """
+        logger.debug("LinkedMediaHelper.get_gfycat_image(img_url=%s)", img_url)
         gfycat_url = ""
         file_path = self.save_dir + "/"
         try:
             gfycat_name = os.path.basename(urlsplit(img_url).path)
+            logger.debug(
+                "LinkedMediaHelper.get_gfycat_image(...) - gfycat_name=%s", gfycat_name
+            )
             async with aiohttp.ClientSession(raise_for_status=True) as session:
                 async with session.get(url=img_url) as response:
-                    soup = BeautifulSoup(response.text, "lxml")
+                    logger.debug(
+                        "LinkedMediaHelper.get_gfycat_image(...) - response.status=%s",
+                        response.status,
+                    )
+                    soup = BeautifulSoup(await response.text(), "html5lib")
+                    logger.debug(
+                        "LinkedMediaHelper.get_gfycat_image(...) - 'cooked soup'"
+                    )
 
-            for tag in soup.find_all("source", src=True):
+            tags = soup.find_all("source", src=True)
+            logger.debug("LinkedMediaHelper.get_gfycat_image(...) - tags=%s", tags)
+
+            for tag in tags:
+                logger.debug("LinkedMediaHelper.get_gfycat_image(...) - tag=%s", tag)
                 src = tag["src"]
-                if "giant" in src and "mp4" in src:
+                if "thumbs" in src:
+                    continue
+                if "mp4" in src:
                     gfycat_url = src
+
+            logger.debug(
+                "LinkedMediaHelper.get_gfycat_image(...) - gfycat_url=%s", gfycat_url
+            )
             file_path += gfycat_name + ".mp4"
+            logger.debug(
+                "LinkedMediaHelper.get_gfycat_image(...) - file_path=%s", file_path
+            )
         except (
             aiohttp.ClientError,
-            GfycatClientError,
             TypeError,
         ) as gfycat_error:
             logger.error("Error downloading Gfycat link: %s", gfycat_error)
@@ -606,8 +567,6 @@ class LinkedMediaHelper:
             logger.debug("Empty Gfycat URL; no attachment to download")
             return None
 
-        logger.debug("Downloading Gfycat at URL %s to %s", gfycat_url, file_path)
-
         saved_paths = await get_file(
             img_url=gfycat_url,
             file_path=file_path,
@@ -689,7 +648,7 @@ class LinkedMediaHelper:
                 if len(tasks) == max_images:
                     break
 
-        rprint("Donwloading images from reddit gallery")
+        print("Donwloading images from reddit gallery")
         file_paths = await asyncio.gather(*tasks)
 
         return file_paths
