@@ -1,17 +1,22 @@
 """This module contains helper classes and methods to assist with determining
 if a reddit post should be published on Mastodon."""
 import configparser
-import logging
 import sys
 from dataclasses import dataclass
+from logging import DEBUG
+from logging import Formatter
+from logging import getLogger
+from logging import StreamHandler
+from logging.handlers import TimedRotatingFileHandler
 from typing import Any
 from typing import List
 from typing import Optional
 from typing import Type
 from typing import TypeVar
+from typing import Union
 
 import aiosqlite
-from rich.logging import RichHandler
+from typing_extensions import Final
 
 from . import __display_name__
 from . import POST_RECORDER_SQLITE_DB
@@ -19,7 +24,7 @@ from . import POST_RECORDER_SQLITE_DB
 PR = TypeVar("PR", bound="PostRecorder")
 ConfigClass = TypeVar("ConfigClass", bound="Configuration")
 
-logger = logging.getLogger(__display_name__)
+logger = getLogger(__display_name__)
 
 
 class PostRecorder:
@@ -27,6 +32,8 @@ class PostRecorder:
     checking against the log of published content to determine if a post would
     be a duplicate."""
 
+    LAST_POST_TS: Final[str] = "last-post-timestamp"
+
     def __init__(self: PR, history_db_dir: str = ".") -> None:
         self.history_db_file = f"{history_db_dir.rstrip('/')}/{POST_RECORDER_SQLITE_DB}"
         self.history_db: Optional[aiosqlite.Connection] = None
@@ -48,6 +55,15 @@ class PostRecorder:
             "(checksum TEXT PRIMARY KEY) "
             "WITHOUT ROWID"
         )
+        await self.history_db.execute(
+            "CREATE TABLE IF NOT EXISTS settings "
+            "(key TEXT PRIMARY KEY, value) "
+            "WITHOUT ROWID"
+        )
+        await self.history_db.execute(
+            "INSERT OR IGNORE INTO settings (key, value) VALUES (:key, :value)",
+            {"key": PostRecorder.LAST_POST_TS, "value": 0},
+        )
         await self.history_db.commit()
 
     async def duplicate_check(self: PR, identifier: str) -> bool:
@@ -66,15 +82,17 @@ class PostRecorder:
                 Mastodon
                 True if "identifier" has been found in log of content.
         """
+        logger.debug("PostRecorder.duplicate_check(identifier=%s)", identifier)
 
         if self.history_db is None:
-            return False
+            raise AssertionError("Have you called db_init() first?")
 
         # check for reddit_id
         cursor = await self.history_db.execute(
             "SELECT * FROM post where id=:id", {"id": identifier}
         )
         if await cursor.fetchone():
+            logger.debug("PostRecorder.duplicate_check(...) - returns True")
             return True
 
         # check for Shared URL
@@ -82,6 +100,7 @@ class PostRecorder:
             "SELECT * FROM share where url=:url", {"url": identifier}
         )
         if await cursor.fetchone():
+            logger.debug("PostRecorder.duplicate_check(...) - returns True")
             return True
 
         # check for attachment hash
@@ -89,8 +108,10 @@ class PostRecorder:
             "SELECT * FROM hash where checksum=:checksum", {"checksum": identifier}
         )
         if await cursor.fetchone():
+            logger.debug("PostRecorder.duplicate_check(...) - returns True")
             return True
 
+        logger.debug("PostRecorder.duplicate_check(...) - returns False")
         return False
 
     async def log_post(
@@ -110,8 +131,15 @@ class PostRecorder:
                 Checksum of media attachment that was shared on Mastodon.
                 This enables checking for duplicate media even if file has been renamed.
         """
+        logger.debug(
+            "PostRecorder.log_post(reddit_id=%s ,shared_url= %s, checksum= %s)",
+            reddit_id,
+            shared_url,
+            check_sum,
+        )
+
         if self.history_db is None:
-            return
+            raise AssertionError("Have you called db_init() first?")
 
         if reddit_id:
             await self.history_db.execute("INSERT INTO post VALUES (?)", (reddit_id,))
@@ -121,6 +149,58 @@ class PostRecorder:
             await self.history_db.execute("INSERT INTO hash VALUES (?)", (check_sum,))
         await self.history_db.commit()
 
+    async def get_setting(
+        self: PR,
+        key: str,
+    ) -> Any:
+        """Retrieves a setting from database.
+
+        :param self: Post Recorded instance
+        :param key: Key to setting stored in DB
+        :type key: str
+
+        :return: Value of setting. This could be an int, str, or float
+        """
+        logger.debug("PostRecorder.get_settings(key=%s)", key)
+        if self.history_db is None:
+            raise AssertionError("Have you called db_init() first?")
+
+        cursor = await self.history_db.execute(
+            "SELECT value FROM settings WHERE key=:key", {"key": key}
+        )
+        row = await cursor.fetchone()
+        if row is None:
+            return None
+
+        logger.debug("PostRecorder.get_settings(...) returns: %s", row[0])
+        return row[0]
+
+    async def save_setting(
+        self: PR,
+        key: str,
+        value: Union[int, str, float],
+    ) -> None:
+        """Retrieves a setting from database.
+
+        :param self: Post Recorded instance
+        :param key: Key to setting stored in DB
+        :type key: str
+        :param value: Value to store as a setting
+        :type key: Union[int, str, float]
+
+        :return: None
+        """
+        logger.debug("PostRecorder.save_settings(key=%s, value=%s)", key, value)
+        if self.history_db is None:
+            raise AssertionError("Have you called db_init() first?")
+
+        await self.history_db.execute(
+            "INSERT OR REPLACE INTO settings (key, value) VALUES (:key, :value)",
+            {"key": key, "value": value},
+        )
+
+        await self.history_db.commit()
+
     async def close_db(self: PR) -> None:
         """Closes db connection."""
         if self.history_db:
@@ -232,13 +312,17 @@ class Configuration:
         """Loads aor queries config parameters from user, as necessary."""
 
         if debug_log:
-            file_log_formatter = logging.Formatter(
+            file_log_formatter = Formatter(
                 "%(asctime)s %(name)s[%(process)d] %(levelname)s %(message)s",
                 datefmt="%H:%M:%S",
             )
-            file_handler = logging.FileHandler(filename=debug_log)
+            file_handler = TimedRotatingFileHandler(
+                filename=debug_log,
+                backupCount=7,
+                when="midnight",
+            )
             file_handler.setFormatter(file_log_formatter)
-            file_handler.setLevel(logging.DEBUG)
+            file_handler.setLevel(DEBUG)
             logger.addHandler(file_handler)
 
         # Make sure config file exists
@@ -343,11 +427,11 @@ class Configuration:
         if config["BotSettings"]["LogLevel"]:
             log_level = config["BotSettings"]["LogLevel"]
 
-        std_out_formatter = logging.Formatter(
+        std_out_formatter = Formatter(
             "%(name)s[%(process)d] %(levelname)s %(message)s",
             datefmt="%H:%M:%S",
         )
-        std_out_handler = RichHandler()
+        std_out_handler = StreamHandler(sys.stdout)
         std_out_handler.setFormatter(std_out_formatter)
         std_out_handler.setLevel(log_level)
         logger.addHandler(std_out_handler)
