@@ -1,12 +1,12 @@
 """This module is a collection of classes and methods to perform the actual
 posting of content to Mastodon."""
-# pylint: disable=too-few-public-methods
-# With two private methods it is still nice to have these in their own module
+import asyncio
 import logging
 import os
 import sys
 from dataclasses import dataclass
 from typing import Any
+from typing import AsyncIterator
 from typing import Dict
 from typing import List
 from typing import Type
@@ -18,11 +18,9 @@ import magic
 from minimal_activitypub.client_2_server import ActivityPub
 from minimal_activitypub.client_2_server import ActivityPubError
 from minimal_activitypub.client_2_server import ApiError
-from rich import print as rprint
 from tqdm.asyncio import tqdm
 
 from . import CLIENT_WEBSITE
-from . import PROGRESS_BAR_FORMAT
 from . import USER_AGENT
 from .collect import LinkedMediaHelper
 from .collect import MediaAttachment
@@ -33,10 +31,14 @@ from .control import MastodonConfig
 from .control import PromoConfig
 from .control import Secret
 
+# pylint: disable=too-few-public-methods
+# With two private methods it is still nice to have these in their own module
+
 
 logger = logging.getLogger("Tootbot")
 
 MP = TypeVar("MP", bound="MastodonPublisher")
+FileManagerClass = TypeVar("FileManagerClass", bound="FileManager")
 
 SECRETS_FILE = "mastodon.secret"  # pragma: allowlist secret
 
@@ -116,10 +118,10 @@ class MastodonPublisher:
         if not os.path.exists(secrets_file):
             # If the secret file doesn't exist,
             # it means the setup process hasn't happened yet
-            rprint("Mastodon API keys not found. (See wiki for help).")
+            print("Mastodon API keys not found. (See wiki for help).")
             user_name = input("[ .. ] Enter email address for Mastodon account: ")
             password = input("[ .. ] Enter password for Mastodon account: ")
-            rprint("Generating login key for Mastodon...")
+            print("Generating login key for Mastodon...")
             try:
                 async with aiohttp.ClientSession() as session:
                     access_token = await ActivityPub.get_auth_token(
@@ -139,7 +141,7 @@ class MastodonPublisher:
                     await mastodon.determine_instance_type()
                     user_info = await mastodon.verify_credentials()
                 mastodon_username = user_info["username"]
-                rprint(
+                print(
                     f"Successfully authenticated on "
                     f"{mastodon_domain} as @{mastodon_username}"
                 )
@@ -148,7 +150,7 @@ class MastodonPublisher:
                     await file.write("\n")
                     await file.write(mastodon.instance)
                     mastodon_secrets.client_secret = access_token
-                rprint(
+                print(
                     f"Mastodon login information now stored in " f"{secrets_file} file"
                 )
 
@@ -242,7 +244,7 @@ class MastodonPublisher:
                         sensitive=self.mastodon_config.media_always_sensitive,
                         spoiler_text=spoiler,
                     )
-                    rprint(f'Posted to Mastodon: "{caption}" (at {toot["url"]})')
+                    print(f'Posted to Mastodon: "{caption}" (at {toot["url"]})')
 
                     # Log the toot
                     await self.bot.post_recorder.log_post(
@@ -282,43 +284,36 @@ class MastodonPublisher:
         if len(attachments.media_paths) == 0:
             return media_ids
 
-        title = "Uploading attachments"
-        for checksum, media_path in tqdm(
-            attachments.media_paths.items(),
-            desc=f"{title:.<60}",
-            total=len(attachments.media_paths),
-            unit="attachment",
-            ncols=120,
-            bar_format=PROGRESS_BAR_FORMAT,
-        ):
-            try:
-                mime_type = magic.from_file(media_path, mime=True)
-                logger.debug(
-                    "Media %s (%s) with checksum: %s",
-                    media_path,
-                    mime_type,
-                    checksum,
+        print("Uploading attachments")
+        tasks: List[Any] = []
+        for checksum, media_path in attachments.media_paths.items():
+            mime_type = magic.from_file(media_path, mime=True)
+            logger.debug(
+                "MastodonPublisher._post_attachements - Media %s (%s) with checksum: %s",
+                media_path,
+                mime_type,
+                checksum,
+            )
+            attachment = FileManager(file_name=media_path)
+            tasks.append(
+                self.instance.post_media(
+                    file=attachment.file_reader(),
+                    mime_type=mime_type,
                 )
-                async with aiofiles.open(file=media_path, mode="rb") as upload:
-                    media = await self.instance.post_media(
-                        file=upload,
-                        mime_type=mime_type,
-                    )
-                logger.debug(
-                    "MastodonPublisher._post_attachments - uploaded: %s", media_path
-                )
-                logger.debug("MastodonPublisher._post_attachments - result: %s", media)
+            )
 
-                # Log the media upload
-                await self.bot.post_recorder.log_post(check_sum=checksum)
+            # Log the media upload
+            await self.bot.post_recorder.log_post(check_sum=checksum)
+        try:
+            medias = await asyncio.gather(*tasks)
+            for media in medias:
+                logger.debug("MastodonPublisher._post_attachments - result: %s", media)
                 media_ids.append(media.get("id"))
-            except (ActivityPubError, TypeError) as error:
-                logger.debug(
-                    "Error when uploading media %s (%s): %s",
-                    media_path,
-                    mime_type,
-                    error,
-                )
+        except (ActivityPubError, TypeError) as error:
+            logger.debug(
+                "MastodonPublisher._post_attachements - Error when uploading media: %s",
+                error,
+            )
 
         logger.debug("MastodonPublisher._post_attachments - media_ids: %s", media_ids)
         return media_ids
@@ -333,6 +328,13 @@ class MastodonPublisher:
             posted on Mastodon
         """
         # Build a list of checksums for files that have already been posted earlier
+        logger.debug(
+            "MastodonPublisher._remove_posted_earlier(attachments=%s)", attachments
+        )
+        logger.debug(
+            "MastodonPublisher._remove_posted_earlier(...) - number of attachements %s",
+            len(attachments.media_paths),
+        )
         checksums = []
         for checksum in attachments.media_paths:
             logger.debug(
@@ -344,8 +346,50 @@ class MastodonPublisher:
                 checksums.append(checksum)
             # Check for duplicate of attachment sha256
             elif await self.bot.post_recorder.duplicate_check(checksum):
-                logger.debug("Media with checksum %s has already been posted", checksum)
+                logger.debug(
+                    "MastodonPublisher._remove_posted_earlier(...) - "
+                    "Media with checksum %s has already been posted",
+                    checksum,
+                )
                 checksums.append(checksum)
         # Remove all empty or previously posted images
         for checksum in checksums:
             attachments.destroy_one_attachment(checksum)
+
+        logger.debug(
+            "MastodonPublisher._remove_posted_earlier(...) - number of attachements after check %s",
+            len(attachments.media_paths),
+        )
+
+
+class FileManager:
+    """Class to wrap around file reads with a tqdm progress bar."""
+
+    def __init__(
+        self: FileManagerClass,
+        file_name: str,
+        chunk_size: int = 64 * 1024,
+    ) -> None:
+        self.name = file_name
+        self.chunk_size = chunk_size
+        self.size = os.path.getsize(self.name)
+        self.pbar = tqdm(
+            total=os.path.getsize(filename=self.name),
+            desc=f"{self.name:.<60}",
+            unit="B",
+            unit_scale=True,
+            unit_divisor=1024,
+            ncols=120,
+            leave=True,
+        )
+
+    async def file_reader(self: FileManagerClass) -> AsyncIterator[bytes]:
+        """file_reader returns file contents in chunks of bytes and updates an
+        associated progress bar."""
+        async with aiofiles.open(self.name, "rb") as file:
+            chunk = await file.read(self.chunk_size)
+            while chunk:
+                self.pbar.update(len(chunk))
+                yield chunk
+                chunk = await file.read(self.chunk_size)
+            self.pbar.close()
